<?php

namespace Drupal\smtp_mail_auth_handler\Plugin\WebformHandler;

use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\Plugin\WebformHandler\EmailWebformHandler;
use Drupal\webform\WebformSubmissionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * SMTP Mail Authentication Webform Handler.
 *
 * @WebformHandler(
 *   id = "smtp_mail_auth_handler",
 *   label = @Translation("SMTP Mail Authentication"),
 *   category = @Translation("Notification"),
 *   description = @Translation("Send an email with custom SMTP credentials."),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_UNLIMITED,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 *   submission = \Drupal\webform\Plugin\WebformHandlerInterface::SUBMISSION_OPTIONAL,
 * )
 */
class SmtpMailAuthHandler extends EmailWebformHandler {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    /** @var static $instance */
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->entityTypeManager = $container->get('entity_type.manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'smtp_credentials_key' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['smtp_credentials'] = [
      '#type' => 'details',
      '#title' => $this->t('SMTP Credentials'),
      '#open' => TRUE,
      '#weight' => -10,
    ];

    $form['smtp_credentials']['smtp_credentials_key'] = [
      '#type' => 'key_select',
      '#title' => $this->t('SMTP Credentials Key'),
      '#description' => $this->t('Select the key containing the SMTP credentials to use for this webform.'),
      '#default_value' => $this->configuration['smtp_credentials_key'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    $this->configuration['smtp_credentials_key'] = $form_state->getValue([
      'smtp_credentials',
      'smtp_credentials_key',
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary() {
    $summary = parent::getSummary();

    if (isset($summary['#settings'])) {
      if (!empty($this->configuration['smtp_credentials_key'])) {
        $smtp_key = $this->configuration['smtp_credentials_key'];

        $key = $this->entityTypeManager->getStorage('key')->load($smtp_key);
        $smtp_key_title = $key ? $key->label() : $smtp_key;

        $summary['#settings']['smtp_credentials_key'] = $this->t('@key', ['@key' => $smtp_key_title]);
      }
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function sendMessage(WebformSubmissionInterface $webform_submission, array $message) {
    // Add the SMTP credentials key to the message parameters.
    if (!empty($this->configuration['smtp_credentials_key'])) {
      $message['params']['smtp_credentials_key'] = $this->configuration['smtp_credentials_key'];
    }

    return parent::sendMessage($webform_submission, $message);
  }

}
